//
//	1.BareBone
//
//  As simple as a shader gets!

float4x4 worldIT;  // World Inverse Transpose
float4x4 world;
float4x4 view;
float4x4 projection;


// goverened by SetFVF!
struct VertexShaderInput
{
	float3 Position : POSITION;
	float3 Normal : NORMAL;
	float2 TexCoord : TEXCOORD0;  // input UV, not used here
};

struct VertexShaderOutput
{
     float4 Position : POSITION;      // MUST have this
     float2 TexCoords : TEXCOORD0;    // UV for pixel shader, not used in this case
     float3 WorldNormal : TEXCOORD1;  // will be linearly interpolated
};

VertexShaderOutput SimpleVS(VertexShaderInput input)
{
    VertexShaderOutput output;
    
    // world view projection matrix
    float4x4 wvp = mul(mul(world, view), projection);
     
    // transform the input position to the output
    output.Position = mul(float4(input.Position, 1.0), wvp);

    // Notice, we use the world-inverse-transpose for the normval vector
    output.WorldNormal =  mul(input.Normal, worldIT);
    
    // simply pass the UVCoordinate down the pipe to the pixel shader
    output.TexCoords = input.TexCoord;

    return output;
}

float4 SimplePS(VertexShaderOutput input) : COLOR
{
    // Not using anything
    float4 color = {0.0f, 0.0f, 0.0f, 0.0f};
	
	color.r = input.TexCoords.x;
	color.g = input.TexCoords.y;
	color.b = 0.0;
    color.a = 1.0;
    return color;
}

technique SimpleShader
{
    pass P0
    {
        VertexShader = compile vs_3_0 SimpleVS();
        PixelShader = compile ps_3_0 SimplePS();
    }
}