//
//	UWB_LightingEffect
//
//  Single-pass shader for multiple lights based on the D3D9 lighting specifications. 
//	* Requires shader version 3_0
//
// All normals and diretional vectors are assumed to be normalized!

float4x4 worldIT;  // World Inverse Transpose
float4x4 world;
float4x4 view;
float4x4 projection;


struct Light
{
	float4 diffuse;
	float3 position;
};

Light myLight;

struct Material
{
	float4 diffuse;
	float4 emissive;
};

Material myMaterial;

texture textureMap;
bool textureEnable;

sampler2D textureSampler = sampler_state
{
    Texture = <textureMap>;
    MagFilter = Linear;
    MinFilter = Linear;
};

struct VertexShaderInput
{
	float3 Position : POSITION;
	float3 Normal : NORMAL;
	float2 TexCoord : TEXCOORD0;

};

struct VertexShaderOutput
{
     float4 Position : POSITION;
     float2 TexCoords : TEXCOORD0;
     float3 WorldNormal : TEXCOORD1;
     float3 WorldPosition : TEXCOORD2;
};

VertexShaderOutput NdotLVS(VertexShaderInput input)
{
    VertexShaderOutput output;
    
    // world view projection matrix
    float4x4 wvp = mul(mul(world, view), projection);
     
    // transform the input position to the output
    output.Position = mul(float4(input.Position, 1.0), wvp);

    // Again, we see WorldNormal being transformed by inverse-transpose
    output.WorldNormal =  mul(input.Normal, worldIT);
    
    // world position of object
    float4 worldPosition =  mul(float4(input.Position, 1.0), world);
    output.WorldPosition = worldPosition / worldPosition.w;	
    
    // texture coordinate simply gets forwarded
    output.TexCoords = input.TexCoord;

    return output;
}


float4 NdotLPS(VertexShaderOutput input) : COLOR
{
    // comput light diretion
    float3 lightVector = myLight.position - input.WorldPosition;
    lightVector = normalize(lightVector);
    
    // initialize resulting color
    float4 color = myMaterial.emissive;
    float4 texColor = {0.0f, 0.0f, 0.0f, 0.0f};
	
    if (textureEnable) {
	    texColor = tex2D(textureSampler, input.TexCoords);
	    color += dot(input.WorldNormal, lightVector) * myLight.diffuse * texColor;
    } else {
        color += dot(input.WorldNormal, lightVector) * myLight.diffuse * myMaterial.diffuse;
    }
    
    return color;
}

technique SimpleNdotL
{
    pass P0
    {
        VertexShader = compile vs_3_0 NdotLVS();
        PixelShader = compile ps_3_0 NdotLPS();
    }
}