#include "stdafx.h"
#include "TutorialApp.h"
#include "TutorialDlg.h"

// DEBUG_NEW macro allows MFC applications to determine memory leak locations in debug builds
#ifdef _DEBUG
  #define new DEBUG_NEW
#endif


CTutorialDlg::CTutorialDlg(CWnd* pParent)
: CDialog(CTutorialDlg::IDD, pParent)
, m_SpotOn(FALSE)
, m_DirOn(FALSE)
, m_PtOn(FALSE)
{
  m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CTutorialDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  DDX_Control(pDX, IDC_SLIDER_CAMERA_X, m_slider_camera_x);
  DDX_Control(pDX, IDC_SLIDER_CAMERA_Y, m_slider_camera_y);
  DDX_Control(pDX, IDC_SLIDER_CAMERA_Z, m_slider_camera_z);
  DDX_Control(pDX, IDC_SLIDER_LOOKAT_X, m_slider_lookat_x);
  DDX_Control(pDX, IDC_SLIDER_LOOKAT_Y, m_slider_lookat_y);
  DDX_Control(pDX, IDC_SLIDER_LOOKAT_Z, m_slider_lookat_z);
  DDX_Control(pDX, IDC_SLIDER_UPVECTOR, m_slider_upvector);
  DDX_Control(pDX, IDC_SLIDER_UPVECTORZ, m_slider_upvectorz);
  DDX_Check(pDX, IDC_SPOT_LIGHT, m_SpotOn);
  DDX_Check(pDX, IDC_DIRECTIONAL_LIGHT, m_DirOn);
  DDX_Check(pDX, IDC_POINT_LIGHT, m_PtOn);
}

BEGIN_MESSAGE_MAP(CTutorialDlg, CDialog)
  ON_WM_HSCROLL()
  ON_BN_CLICKED(IDC_RESET, OnBnClickedReset)
  ON_BN_CLICKED(IDC_CAMERAZ_0, OnBnClickedCameraZ0)
  ON_BN_CLICKED(IDC_POINT_LIGHT, &CTutorialDlg::OnBnClickedPointLight)
  ON_BN_CLICKED(IDC_DIRECTIONAL_LIGHT, &CTutorialDlg::OnBnClickedDirectionalLight)
  ON_BN_CLICKED(IDC_SPOT_LIGHT, &CTutorialDlg::OnBnClickedSpotLight)
END_MESSAGE_MAP()

// This is called when the dialog is first created and shown.
// It is a good spot to initialize member variables.
BOOL CTutorialDlg::OnInitDialog()
{
  CDialog::OnInitDialog();
  m_GrfxWnd.ReplaceDialogControl(m_hWnd, IDC_GRFX_PLACEHOLDER);

  float pos_x, pos_y, pos_z;
  m_GrfxWnd.GetCameraPosition( pos_x, pos_y, pos_z );

  float look_x, look_y, look_z;
  m_GrfxWnd.GetCameraLookAt( look_x, look_y, look_z );

  float rot, z;
  m_GrfxWnd.GetCameraUpVector(rot, z);

  m_slider_camera_x.Initialize(-30.0f, 30.0f, pos_x);
  m_slider_camera_y.Initialize(-30.0f, 30.0f, pos_y);
  m_slider_camera_z.Initialize(-30.0f, 30.0f, pos_z);

  m_slider_lookat_x.Initialize(-30.0f, 30.0f, look_x);
  m_slider_lookat_y.Initialize(-30.0f, 30.0f, look_y);
  m_slider_lookat_z.Initialize(-30.0f, 30.0f, look_z);

  m_slider_upvector.Initialize(-180.0f, 180.0f, rot);
  m_slider_upvectorz.Initialize(-1.0f, 1.0f, z);

  return TRUE;  // return TRUE  unless you set the focus to a control
}

void CTutorialDlg::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
  // We should check to make sure we know which slider bar is generating the events
  float pos_x, pos_y, pos_z;
  float look_x, look_y, look_z;

  float rot,z;
  rot = m_slider_upvector.GetSliderValue();
  z = m_slider_upvectorz.GetSliderValue();

  pos_x = m_slider_camera_x.GetSliderValue();
  pos_y = m_slider_camera_y.GetSliderValue();
  pos_z = m_slider_camera_z.GetSliderValue();

  look_x = m_slider_lookat_x.GetSliderValue();
  look_y = m_slider_lookat_y.GetSliderValue();
  look_z = m_slider_lookat_z.GetSliderValue();

  m_GrfxWnd.SetCameraPosition( pos_x, pos_y, pos_z );
  m_GrfxWnd.SetCameraLookAt( look_x, look_y, look_z );
  m_GrfxWnd.SetCameraUpVector(rot, z);
  m_GrfxWnd.Invalidate();

  CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
}


void CTutorialDlg::OnBnClickedReset()
{
  float rot, z;
  float pos_x, pos_y, pos_z;
  float look_x, look_y, look_z;

  m_GrfxWnd.ResetCameraParameters();  
  m_GrfxWnd.GetCameraPosition( pos_x, pos_y, pos_z );
  m_GrfxWnd.GetCameraLookAt( look_x, look_y, look_z );
  m_GrfxWnd.GetCameraUpVector(rot, z);

  m_slider_camera_x.SetSliderValue(pos_x);
  m_slider_camera_y.SetSliderValue(pos_y);
  m_slider_camera_z.SetSliderValue(pos_z);

  m_slider_lookat_x.SetSliderValue(look_x);
  m_slider_lookat_y.SetSliderValue(look_y);
  m_slider_lookat_z.SetSliderValue(look_z);

  m_slider_upvector.SetSliderValue(rot);
  m_slider_upvectorz.SetSliderValue(z);

  m_GrfxWnd.Invalidate();
}

void CTutorialDlg::OnBnClickedCameraZ0()
{
  float pos_x, pos_y, pos_z;
  m_GrfxWnd.GetCameraPosition( pos_x, pos_y, pos_z );
  m_slider_camera_z.SetSliderValue(0.0f);
  m_GrfxWnd.SetCameraPosition(pos_x, pos_y, 0.0f);
  m_GrfxWnd.Invalidate();
}
void CTutorialDlg::OnBnClickedPointLight()
{
  UpdateData(TRUE);
  m_GrfxWnd.PointOn(m_PtOn);
  m_GrfxWnd.Invalidate();

}

void CTutorialDlg::OnBnClickedDirectionalLight()
{
    UpdateData(TRUE);
    m_GrfxWnd.DirOn(m_DirOn);
    m_GrfxWnd.Invalidate();
}

void CTutorialDlg::OnBnClickedSpotLight()
{
    UpdateData(TRUE);
    m_GrfxWnd.SpotOn(m_SpotOn);
    m_GrfxWnd.Invalidate();
}
